unit FrameDBase;

{$mode ObjFPC}{$H+}

// also see
// --------
// https://wiki.lazarus.freepascal.org/Frames/de
// https://wiki.freepascal.org/Lazarus_Tdbf_Tutorial/de


interface

uses
  Classes, SysUtils, Forms, Controls, Menus, DBGrids, StdCtrls, LazFileUtils,
  DBCtrls, DB, dbf;


const
  CountriesFilename = 'countries.dbf';


type

  { TFrame1 }

  TFrame1 = class(TFrame)
    BSortByCapital: TButton;
    BSortByCountry: TButton;
    BSortByID: TButton;
    dbfSource: TDataSource;
    grid: TDBGrid;
    Navigator: TDBNavigator;
    procedure BSortByCapitalClick(Sender: TObject);
    procedure BSortByCountryClick(Sender: TObject);
    procedure BSortByIDClick(Sender: TObject);
  private
    { private declarations }
    CountriesDbf: TDbf;
  public
    { public declarations }
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;
  end;


implementation

{$R *.lfm}

constructor TFrame1.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);

  CountriesDbf:= TDbf.Create(Self);
  CountriesDbf.TableName:= CountriesFilename;

  if not FileExistsUTF8( CountriesFilename) then
  with CountriesDbf do
    begin
      TableLevel := 7;
      Exclusive  := True;
      FieldDefs.Add( 'ID', ftAutoInc, 0, True);
      FieldDefs.Add( 'CountryName', ftString, 25, True);
      FieldDefs.Add( 'Capital', ftString, 25, True);
      CreateTable;
      Open;

      Append;
      // Fields[0].AsInteger:= 100;  // Do not fill 'ID' because its type is ftAutoInc
      fields[1].AsString:= 'France';
      Fields[2].AsString:= 'Paris';
      Post;

      Append;
      // Fields[0].AsInteger:= 101;
      fields[1].AsString:= 'Egypt';
      Fields[2].AsString:= 'Cairo';
      Post;

      Append;
      // Fields[0].AsInteger:= 102;
      fields[1].AsString:= 'Indonesia';
      Fields[2].AsString:= 'Jakarta';
      Post;

      Append;
      // Fields[0].AsInteger:= 103;
      fields[1].AsString:= 'Austria';
      Fields[2].AsString:= 'Vienna';
      Post;

      AddIndex( 'idxByID', 'ID', [ixPrimary,ixUnique]);
      AddIndex( 'idxByCountry', 'CountryName', [ixCaseInsensitive]);
      AddIndex( 'idxByCapital', 'Capital', [ixCaseInsensitive]);
    end;

  CountriesDbf.Open;
  dbfSource.DataSet    := CountriesDbf;
  grid.DataSource      := dbfSource;
  Navigator.DataSource := dbfSource;
end;

destructor TFrame1.Destroy;
begin
  with CountriesDbf do
    begin
      CloseIndexFile( 'idxByCapital');
      CloseIndexFile( 'idxByCountry');
      CloseIndexFile( 'idxByID');
    end;
  CountriesDbf.Close;

  inherited Destroy;
end;

procedure TFrame1.BSortByCapitalClick(Sender: TObject);
begin
  CountriesDbf.IndexName:= 'idxByCapital';
end;

procedure TFrame1.BSortByCountryClick(Sender: TObject);
begin
  CountriesDbf.IndexName:= 'idxByCountry';
end;

procedure TFrame1.BSortByIDClick(Sender: TObject);
begin
  CountriesDbf.IndexName:= 'idxByID';
end;


end.

